#!/usr/bin/perl -s
#********************************************************************************
#                                                                               *
#                       f2html.pl                                               *
#                                                                               *
#********************************************************************************
# Copyright (C) 2000 by Nikolei Steinhage.   All Rights Reserved.               *
#********************************************************************************
# This program is free software; you can redistribute it and/or                 *
# modify it under the terms of the GNU General Public                           *
# License as published by the Free Software Foundation; either                  *
# version 2 of the License, or (at your option) any later version.              *
#                                                                               *
# This program is distributed in the hope that it will be useful,               *
# but WITHOUT ANY WARRANTY; without even the implied warranty of                *
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU             *
# General Public License for more details.                                      *
#                                                                               *
# You should have received a copy of the GNU Library General Public             *
# License along with this program/script; if not, write to the Free             *
# Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.            *
#********************************************************************************
# Bugs: Doubtless many. Feel free to contact the author, but remember:          *
#       I do not promise anything.                                              *
#       nikolei.steinhage@uni-bielefeld.de                                      *
#********************************************************************************
# Disclaimer: - I do not support Perl, html, or any OS                          *
#             - I do not promise that this program does anything                *
#             - this program may distroy all your data                          *
#             - I cannot promise that this program will not distroy             *
#               all human civilization                                          *
#********************************************************************************
# Das fortwaerende Dasein des Menschengeschlechts ist bloss ein Beweis          *
# der Geilheit desselben.         ATHUR SCHOPENHAUER                            *
# Translation: The continous existence of mankind is only a proof of their      *
# contempt.                                                                     *   
#********************************************************************************

use MP3::Info;
use Getopt::Long;


#********************************************************************************
# global variables i need

my @mp3v;                       # directories for depth search
my $htmlfile;                   # html file (at the moment only one)
my $m3ufile;                    # playlistfile
my $dbfile;                     # database file
my @filelist;                   # array to store file and infor seperated by the string "@@@"
my $hexfile;                    # file the hextable ist stored in
my %hexhash;                    # hash to store hexreplacements
my $httpaddress;                # address of httpd
my $seperatepath;               # path for seperate html files
my @checkext;                   # string to keep fileext for checks
my $os="unix";                  # default os
my @execs;                      # array for execs
my $depth;                      # depth of dir elements to keep
my %replhash;                   # hash for url replacement
my $replacefile;                # file that contains strings to be replaced at the beginning of 
                                # each url; the form is for example:  c:\mp3\A /mp3C and then a new line
my %sum_files;                  # hash for number of files for every beginning character
my %sum_sizes;                  # hash for sum of sizes for every character
my $statfile;                   # file for html-statistics

#********************************************************************************
# flags to define for controlling runnig of the program
my $MP3FLAG=0;                  # get mp3info
my $TAGRMFLAG=0;                # remove tags
my $HTTPFLAG=0;                 # correct path and character
my $M3UFLAG=0;                  # write m3u file
my $HTMLFLAG=0;                 # write html file
my $DBFLAG=0;                   # write db file
my $OSFLAG=0;                   # set os, default = unix
my $REMEXTFLAG=0;               # remove file extensions
my $SEPERATEFLAG=0;             # write seperate html file for every character
my $CHECKFLAG=0;                # check for file extension like "mp3"
my $FILESIZEFLAG=0;             # include filesize in html output
my $DEPTHFLAG=0;                # use only last n elements of dir/file
my $REPLACEFLAG=0;              # replace pattern given by replacementfile
my $HEXFLAG=0;                  # replace characters by their hex
my $NOCSFLAG=0;                 # do sorting not case sensitive
my $STATFILEFLAG=0;             # write statistics file

#********************************************************************************
# counter and summaries 


#********************************************************************************
# so lets start the program
# here i also have to take into account the parameters in if contructions 
# for different user wishes

&Mainprog;


#********************************************************************************
# here is the main prog

sub Mainprog {
  
  $|=1; # turn of I/O buffering

  &Getparameters;

  if($HEXFLAG==1){&Loadhextable($hexfile);}

  if($REPLACEFLAG==1){&Loadreplacementtable($replacefile);}

  &Searchengine;

  if($NOCSFLAG==1){@filelist=sort{ lc($a) cmp lc($b) }@filelist;}
  else{@filelist=sort(@filelist);}

  if($M3UFLAG==1){&Writem3u($m3ufile);}
 
  if($HTMLFLAG==1 && $SEPERATEFLAG==0){&Writehtml($htmlfile);}
 
  if($DBFLAG==1){&Writedb($dbfile);}
 
  if($SEPERATEFLAG==1 && $HTMLFLAG==0){&Writesephtml($seperatepath);}

  if($STATFILEFLAG==1){&Writestatfile($statfile);}

  if(@execs){
    foreach $execs(@execs){
      &Execsys($execs);
    }
  }


  &Message("Job","done");

}

#********************************************************************************
# just make a output on stdout for sysntax etc.

sub Help {
	print <<EOF
file2html.pl - make playlists and html files. Version: 0.3
by Nikolei Steinhage <nikolei.steinhage\@uni-bielefeld.de>  
Latest version at http://alia.soziologie.uni-bielefeld.de/download.  
This program is licensed under Version 2 of the GNU Public License.  
See http://www.gnu.org for details.

Usage: perl file2html.pl [-h] [[-o (win/unix)] [[-t] <file>] [[-m] <file>] 
                         [[-b] <file>] [[-c] <ext> [-c] <ext> ...] 
                         [[-a] <address>] [-r] [-q] [-i] [-e] [[-s] <path>] 
                         [[-j] <file>]
                         [[-p] <file>] [[-y] <file>] [[-w] <file>]
                         [[-d] <directory1> [-d] <directory2> ...]

	-h, --help                  Show this screen and exit       
        -o, --os       (win,unix)   Default "unix", otherwise windows
        -b, --dbfile   <file>       Write database to a file for searching it        
        -m, --m3u      <file>       Write a m3u file
        -t, --html     <file>       Write a html file
        -a, --http     <name>       Define the http address for url
        -r, --remove                Remove id3tags
        -i, --mp3info               Use mp3info for html output
        -e, --ext                   Remove file extensions in html output 
        -f, --filesize              Use filesize for html output
        -c, --check    <ext>        Select files by their extension (s)(e.g. mp3)
        -z, --skip     <number>     Skip  n elements of mount/directories/names
        -p, --conf     <file>       Use a config file
        -w, --hexfile  <file>       File with hexcode for replacements in links
        -q, --nocs                  Do sorting not case sensitive
        -j, --statfile <file>       Write statistics to file
        -d, --dir      <directory>  Define the (multiple) directory(ies) the 
                                    mp3s are stored in   
        -y, --replace  <file>       Name of replacement file; in the file use 
                                    string_1 string_2 to transform string_1 
                                    into string_2; special characters like a 
                                    backslash have to be preceeded by a 
                                    backslash "\\"
        -s, --seperate <path>       Write seperate html files for every 
                                    character 
                                    (this option is a little bit buggy)
        in the config file it is possible to use the command
        x=, exec=      <param>      Execute system command.
EOF
;

	exit 1;
}

#********************************************************************************
# get the parameters and check them 
# here a lot of things have to be done !!!!!!!!!!

sub Getparameters {
  
  my $error; 
  my $conffile;

  $error=&GetOptions(
  "help|h"      => \&Help,
  "os|o:s"      => \$os,
  "dbfile|b:s"  => \$dbfile,
  "m3u|m:s"     => \$m3ufile,
  "html|t:s"    => \$htmlfile,
  "http|a:s"    => \$httpaddress,
  "remove|r"    => \$TAGRMFLAG, # no argument; variable is set to one if true
  "mp3info|i"   => \$MP3FLAG,
  "ext|e"       => \$REMEXTFLAG,
  "seperate|s:s"=> \$seperatepath,
  "dir|d:s"     => \@mp3v,
  "check|c:s"   => \@checkext,
  "filesize|f"  => \$FILESIZEFLAG,
  "conf|p:s"    => \$conffile,
  "skip|z:i"    => \$depth,
  "replace|y:s" => \$replacefile,
  "hexfile|w:s" => \$hexfile,
  "nocs|q"      => \$NOCSFLAG,   # no argument; variale is set to one if true
  "statfile|j:s" => \$statfile,
  );
  
  
  # if a configuration file is specified, then use it!!!
  if($conffile){&Configfile($conffile);}

  # set the flags needed and not set in GetOptions!!!!
  if($os eq "win"){$OSFLAG=1;}
  else{$OSFLAG=0;}                 # default unix else window

  # delete last slash or backslash if existing
  $seperatepath=&Delslash($seperatepath,$OSFLAG);
  foreach $mp3v(@mp3v){$mp3v=&Delslash($mp3v,$OSFLAG);}


  if($httpaddress){$HTTPFLAG=1;}   # set the url to httpaddess for htmlfile, correct slashes for win
  if($hexfile){$HEXFLAG=1;}
  if($dbfile){$DBFLAG=1;}
  if($m3ufile){$M3UFLAG=1;}   
  if($htmlfile){$HTMLFLAG=1;}
  if($seperatepath){$SEPERATEFLAG=1;}
  if(@checkext){$CHECKFLAG=1;}
  if($depth){$DEPTHFLAG=1;}
  if($replacefile){$REPLACEFLAG=1;}
#  if($sepdirs){$SEPDIRSFLAG=1;}
  if($statfile){$STATFILEFLAG=1;}

  if(!@mp3v) {
	  warn "\nTry file2html.pl --help |more\n";
	  exit 1;
  }

  if(!$error) {
          &Help;
          exit 1;
  }

}

#********************************************************************************
# search for files through the directories and get the informations needed

sub Searchengine {
  my @statistics;   # to store file statistics
  my $fileinfos;    # to store file infos
  my $info;         # to store id3tag info
  my $dircount=0;   # to cout directories
  my $name;         # name of current file
  my $CHECKOK;      # if checkext found then 1 

  &Message("Begin","Collecting files");

  foreach(@mp3v){

    chdir($mp3v[$dircount]);
    opendir(ACDIR,$mp3v[$dircount])||die"cannot open $mp3v[$dircount] !";  

INPUT: foreach $name(readdir(ACDIR)){
        $fileinfos="";      # free $fileinfos, because some files might be not mp3

        next INPUT  if $name =~ /^\.|\.\.$/;  # skip upper dirs

        if(-d $name){ 
          if($OSFLAG == 0){push @mp3v, join '',$mp3v[$dircount] , "/" , $name;}	
          else{push @mp3v, join '',$mp3v[$dircount] , "\\" , $name;}
          next INPUT;
        }
  
        else{
          ### new !!!!!!!!!!!! if extension of filename not equal to the 
          ### extension wanted, then skip here and jump back to INPUT: foreach ....
          ### i hope that the extension check is not case sensitive. i haven't checked
          ### it out yet.

#C          next INPUT if($CHECKFLAG==1 && !($name =~ /($checkext)\Z/));
         if($CHECKFLAG==1){
            $CHECKOK=0;
            foreach $checkext(@checkext){
              if($name =~ /\.($checkext)\Z/){
                 $CHECKOK=1;
              }
            }
            next INPUT if($CHECKOK==0);
         }

          @statistics=stat($name);

          if($MP3FLAG==1){
            if($name=~/(.mp3)\Z/){
              $fileinfos=Getmp3info($name);
            }
            else{$fileinfos=" ";}
          }
          else{$fileinfos=" ";}

          
          if($TAGRMFLAG==1){
            if($name=~/(.mp3)\Z/){  #### ueberdenken 
              &Removeid3tag($name);
            }
          }
           push @filelist , join '@@@', $name,$dircount,$statistics[7],$statistics[9],$fileinfos;
           # new ############################### 
           if($NOCSFLAG==1){           
              $sum_files{uc(substr($name,0,1))}++; 
              $sum_sizes{uc(substr($name,0,1))}+=$statistics[7];
           }
           else{
              $sum_files{substr($name,0,1)}++; 
              $sum_sizes{substr($name,0,1)}+=$statistics[7];
           }
        }
    }
    closedir(ACDIR);
    $dircount++;
  }
}


#********************************************************************************
# load  unicode table if necessary:  &Loadunicodetable(filename)

sub Loadhextable {
  my $param = $_[0];   # name of hexfile
  my $a;               # temporary variable
  my $b;               # temporary variable
  my $line;            # temporary, line to be read in

  &Openfile($param,"<",HEXFILE);
  while($line=<HEXFILE>){
    chomp($line);
    ($a,$b)=split/ /,$line,2;
    $hexhash{$a}=$b; 
  }
  &Closefile(HEXFILE);
}



#********************************************************************************
# correct hex codes in links: &Correcthexlinks(name)

sub Correcthexlinks {
  my $param = $_[0];    # paramter of sub routine
  my $j;                # counter
  my $a="";                # temopary varibale
  my $b="";                # array to return;
  my $lenghofparam=0;   # to mack it faster claculate it once


  $lenghofparam=length($param);
  for($j=0;$j<=$lenghofparam;$j++){
    $a=substr($param,$j,1);
    if($hexhash{$a}){$a=$hexhash{$a};}
#    else{}
    $b=$b.$a;
  }
  return $b;
}

#********************************************************************************
# build replacementhash

sub Loadreplacementtable {
  my $param = $_[0];   # name of file
  my $a;               # temporary variable
  my $b;               # temporary variable
  my $line;            # temporary, line to be read in

  &Openfile($param,"<",REPLFILE);

  while($line=<REPLFILE>){
    chomp($line);
    ($a,$b)=split/ /,$line,2;
    $replhash{$a}=$b; 
  }

  &Closefile(REPLFILE);

}



#********************************************************************************
# correct links by replacement file: &Replacelink(name)

sub Replacelink {

  my $param = $_[0];   # name of url
  my @y;      # keys of hash
  my $a;    

  @y=keys %replhash;


  foreach $y(@y){
    $param =~ s/^$y/$replhash{$y}/ge;  
  }

  return $param;  
  
}
 

#********************************************************************************
# delete file extension:  &Reletefileext(name)

sub Deletefileext {
 
  my $param = $_[0];   # parameter of sub routine
  
  $param=~ s/\.*\w*\Z//g;
  
  return $param;
}


#********************************************************************************
# correct path-names for win if html contains http adress; from / to \
# : &Correctpathname(name)

sub Correctpathnames {
  my $param = $_[0];   # parameter of sub routine
  
  $param=~ s/\\/\//g;
  
  return $param;
}

#********************************************************************************
# remove the id3tag: &Removeid3tag(filename)

sub Removeid3tag {
  my $param = $_[0];   # parameter of sub routine
  
  remove_mp3tag($param);
   
}


#********************************************************************************
# get mp3info: &Getmp3info(filename)

sub Getmp3info {
  my $param = $_[0];   # parameter of sub routine
  my $a;               # temporary variable to store fileinfos
  my $info;            # get mp3info

  if($info=get_mp3info($param)||warn"\nfile $param not mp3 file or not ok !!!"){
    if ($info->{VBR} == 1){ 
      # here you can modify the string to adjust the output for the html file
      $a=join '',$info->{BITRATE} , "bits/s VBR, " , $info->{FREQUENCY} , "kHz, " , 
          $info->{MM} , "\'\'" . $info->{SS} , "\'";
    }
    else{
      # here you can modify the string to adjust the output for the html file
      $a=join '',$info->{BITRATE} , "bits/s, " , $info->{FREQUENCY} , "kHz, " , 
          $info->{MM} , "\'\'" . $info->{SS} , "\'";
    }
  }

  return $a;
}


#********************************************************************************
# open filehandle: &Openfile(filename,mode,filehandle)

sub Openfile {
   my $filename = $_[0];       # name of file to open
   my $filemode = $_[1];       # mode: read, write, append etc.
   my $filehandle = $_[2];     # name of filehandle

   open($filehandle, "$filemode $filename")||die"cannot open $filename!";

}



#********************************************************************************
# close filehandle: &Closefile(handle);

sub Closefile {
   my $filehandle = $_[0];     # name of filehandle
   
   close($filehandle)||die"cannot close $filehandle!";
}

#********************************************************************************
# write m3u file: &Writem3u(filename)

sub Writem3u {
  
  my $param = $_[0];               # name of file to write
  my $datastring="";               # string to store the m3u records

  &Openfile($param,">",M3UFILE);

  if($OSFLAG==0){
    foreach $filelist(@filelist){
      my ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$filelist,5;
      $datastring=join '',$datastring,"$mp3v[$position]/$sortfilename\n";
    }
  }
  else{
    foreach $filelist(@filelist){
      my ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$filelist,5;
      $datastring=join '',$datastring,"$mp3v[$position]\\$sortfilename\n";
    }
  }
  print {M3UFILE} "$datastring";
  &Closefile(M3UFILE);
}

#********************************************************************************
# write db file: &Writedb(filename)

sub Writedb {

  my $param = $_[0];               # name of file to write
  my $datastring="";               # string to store the m3u records

  &Openfile($param,">",DBFILE);

  foreach $filelist(@filelist){
    my ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$filelist,5;                     
    $datastring=$datastring . join '@@@',$mp3v[$position],$sortfilename,"$modificationtime\n";
  }

  print {DBFILE} "$datastring";
  &Closefile(DBFILE);
}

#********************************************************************************
# write one html file or

sub Writehtml {

  my $param = $_[0];            # name of file to write
  my $urlname;                  # url name that can be corrected
  my $datastring="";            # string to store the m3u records
  my $shownname;                # name that is visible
  my $firstchar=" ";            # need a character to compare the first char of exery file with
  my $FTFLAG=0;                 # flag
  my $sunoffiles=0;             # sum of files
  my $sumofmegs=0;              # sum of filesizes
  my $keyofhash=0;              # keys for the hashes sum_file and sum_sizes

  &Message("Begin","Preparing html file");

  &Openfile($param,">",HTMLFILE);

  $datastring=join '',$datastring,&Htmlheader;

  foreach $filelist(@filelist){
    my ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$filelist,5;
#    $sumoffiles++;
#    $sumofmegs+=$filesize;    

    $urlname=$mp3v[$position];
    
    if($REPLACEFLAG==1){$urlname=&Replacelink($urlname)};

    if($OSFLAG==1){$urlname=&Correctpathnames(join '',$urlname,"\\",$sortfilename);}
    else{$urlname=join '',$urlname,"/",$sortfilename;}

    if($DEPTHFLAG==1){$urlname=&Keeppath($urlname,$depth);}

    if($HEXFLAG==1){$urlname=&Correcthexlinks($urlname);}

    if($HTTPFLAG==1){$urlname=join '',$httpaddress,"/",$urlname;}

    if($REMEXTFLAG==1){$shownname=&Deletefileext($sortfilename);}
    else{$shownname=$sortfilename;}

    if($FILESIZEFLAG==1){$filesize=join '',int($filesize/1000),"kb";}
    else{$filesize=" ";}

    if($NOCSFLAG==1 & lc(substr($sortfilename,0,1)) ne lc($firstchar)){
      $firstchar=substr($sortfilename,0,1);
      $datastring=join '',$datastring,&Htmlchange(uc($firstchar));
    }
    elsif($NOCSFLAG==0 & substr($sortfilename,0,1) ne $firstchar){ 
     $firstchar=substr($sortfilename,0,1);
     $datastring=join '',$datastring,&Htmlchange($firstchar);
    }

   $datastring= join '',$datastring,&Htmlbody($urlname,$shownname,$filesize,$fileinfos);
  }
  

  @keysofhash = keys %sum_files;
  foreach $keysofhash(@keysofhash){$sumoffiles+=$sum_files{$keysofhash};}
  
  #@keysofhash = keys %sum_sizes; # redundant because should be equal to the one before!
  foreach $keysofhash(@keysofhash){$sumofmegs+=$sum_sizes{$keysofhash};}

  $datastring= join '',$datastring,&Htmlfooter($sumoffiles,int($sumofmegs/1000000),&Date);

  &Message("Begin","Writing html file");
  print {HTMLFILE} "$datastring";
  &Closefile(HTMLFILE);
}

#********************************************************************************
# here is the definition of the html header

sub Htmlheader {

   my @para = $_[0];          # firstchar not needed here
   my $datastring="";         # this variable will contain the html code for the header
   my @y;                     # values of hash  

   @y = keys %sum_files;

   $datastring = ("<html><body><a name=top>");
    
   foreach $y(@y){
      $datastring = join '', $datastring, "<a href=\"\#".$y."\">".$y."</a>";
   }  
   
   $datastring = join '', $datastring, "<br>";

   return $datastring;
}


#********************************************************************************
# here is the definition of the html footer

sub Htmlfooter {
   
   my @params = @_;           # parameters given to this function like sum of files, date etc.
   my $datastring="";         # this variable will contain the html code for the header
 
   $datastring = ("
      <BR><P><a href=\"\#top\">back to top</a>
      </P>Sum of files is $params[0]. 
      <br>Sum of megabytes is $params[1].
      <br>Generated by file2html.pl on $params[2], 
      (c)2000-2001 Nikolei Steinhage </body></html>
    ");

   return $datastring;

}

#********************************************************************************
# here is the definition of the html char change: &Htmlchange(char)

sub Htmlchange {

   my $param = $_[0];         # charcater
   my $datastring="";         # this variable will contain the html code for the header

   $datastring=("<br><P><a href=\"\#top\">back to top</a></P><br><P><a name=$param>$param</a></P>\n");

   return $datastring;

}

#********************************************************************************
# here is the definition of the html body: &Htmlbody(vars)

sub Htmlbody {

   my @params = @_;           # paramteters like filename, http etc.
   my $datastring="";         # this variable will contain the html code for the header

   $datastring=("<br><a href=\"$params[0]\">$params[1]</a> $params[2] $params[3]\n");

  return $datastring;  

}

#********************************************************************************
# time: &Date

sub Date {

  my @months = qw(Jan Feb Mar Apr May Jun Jul Aug Sep Oct Nov Dec); 
  my ($sec,$min,$hour,$mday,$mon,$yearcomp) = localtime(time); 
  my $time = sprintf("%02d/%s/%d, %02d:%02d:%02d", 
                 $mday, $months[$mon], $yearcomp + 1900,
                 $hour, $min, $sec);

  return $time;
   
}

#********************************************************************************
# write seperate html file for every chart: &Writesephtml(path)
# at the moment this sub is a little bit buggy!!! the misc category dos not
# work very well. so you have to wait ;-) stay tune.

sub Writesephtml {
 
  my $htmlpath=$_[0];           # path to write html files: a.html, b.html .....
  my $urlname;                  # url name that can be corrected
  my $datastring="";            # string to store the m3u records
  my $shownname;                # name that is visible
  my $firstchar=" ";            # need a character to compare the first char of exery file with
  my $FTFLAG=0;                 # flag
#  my $sunoffiles=0;             # sum of files
#  my $sumofmegs=0;              # sum of filesizes
  
  
  &Message("Begin","Preparing html files");

  $datastring=$datastring.&Htmlheader;

  foreach $filelist(@filelist){
    my ($sortfilename, $position, $filesize, $modificationtime, $fileinfos) = split/@@@/,$filelist,5;

#    $sumoffiles++;
#    $sumofmegs+=$filesize;

    $urlname=$mp3v[$position];

    if($REPLACEFLAG==1){$urlname=&Replacelink($urlname)};

    if($OSFLAG==1){$urlname=&Correctpathnames(join '',$urlname,"\\",$sortfilename);}
    else{$urlname=join '',$urlname,"/",$sortfilename;}
   
    if($DEPTHFLAG==1){$urlname=&Keeppath($urlname,$depth);}

    if($HEXFLAG==1){$urlname=&Correcthexlinks($urlname);}

    if($HTTPFLAG==1){$urlname=$httpaddress."/".$urlname;}

    if($REMEXTFLAG==1){$shownname=&Deletefileext($sortfilename);}
    else{$shownname=$sortfilename;}

    if($FILESIZEFLAG==1){$filesize=join '',int($filesize/1000),"kb";}
    else{$filesize=" ";}

    if(($NOCSFLAG==1 & uc(substr($sortfilename,0,1)) ne uc($firstchar))| 
       ($NOCSFLAG==0 & substr($sortfilename,0,1) ne $firstchar)){


     if($firstchar ne " "){
       if($NOCSFLAG==1){$datastring=join '', $datastring,&Htmlfooter($sum_files{uc($firstchar)},int($sum_sizes{uc($firstchar)}/1000000),&Date);}
       else{$datastring=join '', $datastring,&Htmlfooter($sum_files{$firstchar},int($sum_sizes{$firstchar}/1000000),&Date);}
    
#        $datastring=join '', $datastring,&Htmlfooter($sumoffiles,int($sumofmegs/1000000),&Date);       
        print {HTMLFILE} $datastring;
        close(HTMLFILE);
        $datastring="";
#        $sumoffiles=0;
#        $sumofmegs=0;
      }

     if($NOCSFLAG==1){$firstchar=uc(substr($sortfilename,0,1));}
     else{$firstchar=substr($sortfilename,0,1);};
        
     if($NOCSFLAG==1){
        if($OSFLAG==1){&Openfile($htmlpath."\\".uc($firstchar).".html",">",HTMLFILE);}
        else{&Openfile($htmlpath."/".uc($firstchar).".html",">",HTMLFILE);}
        $datastring=&Sephtmlheader(uc($firstchar));
     }
     else{
        if($OSFLAG==1){&Openfile($htmlpath."\\".$firstchar.".html",">",HTMLFILE);}
        else{&Openfile($htmlpath."/".$firstchar.".html",">",HTMLFILE);}
        $datastring=&Sephtmlheader($firstchar);
     }


   }
    $datastring=join '', $datastring,&Htmlbody($urlname,$shownname,$filesize,$fileinfos);

  }
  
  if($NOCSFLAG==1){$datastring=join '', $datastring,&Htmlfooter($sum_files{uc($firstchar)},int($sum_sizes{uc($firstchar)}/1000000),&Date);}
  else{$datastring=join '', $datastring,&Htmlfooter($sum_files{$firstchar},int($sum_sizes{$firstchar}/1000000),&Date);}
  
#  $datastring=join '', $datastring,&Htmlfooter($sumoffiles,int($sumofmegs/1000000),&Date);
#  $datastring=join '', $datastring,&Htmlfooter($sum_files{$firstchar},int($sum_sizes{$firstchar}/1000000),&Date);

  &Message("Begin","Preparing html files");  
  print {HTMLFILE} "$datastring";
  &Closefile(HTMLFILE);
  
}


#********************************************************************************
# write html header for seperate html files: &Sephtmlheader(char)
#$datastring=&Sephtmlheader($firstchar);

sub Sephtmlheader {

  my $param=$_[0];      # parameter: should be $firstchar
  my $datastring="";    # datastring to store html code in

  $datastring=("
    <html>
    <body>
    <p>$param</P>
  ");

  return $datastring;    

}


#********************************************************************************
# read parameters from config file: &Configfile(filename)

sub Configfile {
   
  my $param = $_[0];     # name of config file
  my $line;              # one line of file

  &Openfile($param,"<",CONFFILE);
  while($line=<CONFFILE>){
    chomp($line);
    my(@value)=split/=/,$line;

    SWITCH:{

        if($value[0] =~ /^(o)$/ or $value[0] =~ /^(os)/){
          $os=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(b)$/ or $value[0] =~ /^(dbfile)/){
          $dbfile=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(m)$/ or $value[0] =~ /^(m3u)/){
          $m3ufile=$value[1];  
          last SWITCH;
        }

        if($value[0] =~ /^(t)$/ or $value[0] =~ /^(html)/){
          $htmlfile=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(a)$/ or $value[0] =~ /^(http)/){
          $httpaddress=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(r)$/ or $value[0] =~ /^(remove)/){
          $TAGRMFLAG=1; 
          last SWITCH;
        }

        if($value[0] =~ /^(i)$/ or $value[0] =~ /^(mp3info)/){
          $MP3FLAG=1; 
          last SWITCH;
        }

        if($value[0] =~ /^(s)$/ or $value[0] =~ /^(seperate)/){
          $seperatepath=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(c)$/ or $value[0] =~ /^(check)/){
#C          $checkext=$value[1]; 
          push @checkext, $value[1];
          last SWITCH;
        }

        if($value[0] =~ /^(f)$/ or $value[0] =~ /^(filesize)/){
          $FILESIZEFLAG=1; 
          last SWITCH;
        }

        if($value[0] =~ /^(d)$/ or $value[0] =~ /^(dir)/){
          push @mp3v, $value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(x)$/ or $value[0] =~ /^(exec)/){
          push @execs, $value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(z)$/ or $value[0] =~ /^(skip)/){
          $depth=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(y)$/ or $value[0] =~ /^(replace)/){
          $replacefile=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(w)$/ or $value[0] =~ /^(hexfile)/){
          $hexfile=$value[1]; 
          last SWITCH;
        }

        if($value[0] =~ /^(q)$/ or $value[0] =~ /^(nocs)/){
          $NOCSFLAG=1; 
          last SWITCH;
        }

        if($value[0] =~ /^(j)$/ or $value[0] =~ /^(statfile)/){
          $STATFILEFLAG=1; 
          $statfile=$value[1];
          last SWITCH;
        }


    } # end of SWITCH 

  }

  &Closefile(CONFFILE);

}

#********************************************************************************
# execute system commmands: &Execsys(parameter)

sub Execsys {

  my  $param  = $_[0];    # system command

  &Message("Executing",$execs);
  system($param);
}

#********************************************************************************
# only keep n paths: &Keeppath(path,depth)

sub Keeppath {

  my $pathname  = $_[0];    # pathname
  my $depth     = $_[1];    # depth
  my $tempname="";          # temorary variable
  my @elements;             # elements of tempname
  my $elementcount;         # count elemets
  my $i;                    # simple counter

  @elements=split /\//g,$pathname;
  $elementcount=@elements;
  for($i=$depth+1;$i<$elementcount;$i++){
    $tempname=join '/',$tempname,$elements[$i];
  }
  
  return $tempname;
}

#********************************************************************************
# give out a message on stderr: &Message(1st,2nd)

sub Message {
   my $category = $_[0];  # category
   my $message  = $_[1];  # messageinfo

   printf {STDERR} "%10s: %s\n",$category,$message;  

}

#********************************************************************************
# delete / or \ in patname for configs: &Delslash(path,osflag)

sub Delslash {
   my $pathname=$_[0];
   my $os      =$_[1];

   if($os==0){$pathname =~ s/\/\Z//;}
   else{$pathname =~ s/\\\Z//;}

   return $pathname;
}


#********************************************************************************
# write a statistics file: &Writestatfile(filename);

sub Writestatfile {
   
    my $file=$_[0];   # filename
    my $datastring=""; 
    my @y;
    my $sum=0;
    my $temp;

    @y = keys %sum_files;

    &Message("Begin","Writing statistics file");
    &Openfile($file,">",STATOUT);    
    $datastring=join '',$datastring, "<html><body><center><b><u>SOME STATISTICS</u></b><br><br><br>";


    ############## number of files per character
    $datastring=join '',$datastring, "<b>NUMBER OF FILES PER CHARACTER</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$sum_files{$y}."</td>";
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    foreach $y(@y){
      $sum+=$sum_files{$y};
    }
    $datastring=join '',$datastring, "<br><br>";
    ###############


    ############## percentage of files per character
    $datastring=join '',$datastring, "<b>PERCENTAGE OF FILES PER CHARACTER</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
      $temp=sprintf "%5.2f",(($sum_files{$y}/$sum)*100);
#      $datastring=join '',$datastring, "<td>". (($sum_files{$y}/$sum)*100) ."</td>"; # $sum has bee built before
      $datastring=join '',$datastring, "<td>$temp</td>"; # $sum has bee built before
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    $datastring=join '',$datastring, "Overall there are ".$sum." files.<br><br>";
    ###############


    ############## sizes of files per character
    $datastring=join '',$datastring, "<b>SUM OF SIZES PER CHARACTER IN MEGA BYTES</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".int($sum_sizes{$y}/1000000)."</td>";
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    foreach $y(@y){
      $sum+=$sum_sizes{$y};
    }
    $datastring=join '',$datastring, "<br><br>";
    ###############


    ############## percentage of sizes of files per character
    $datastring=join '',$datastring, "<b>PERCENTAGE OF SIZES PER CHARACTER</b><br><table border=1><tr>";
    foreach $y(@y){
      $datastring=join '',$datastring, "<td>".$y."</td>";
    }
    $datastring=join '',$datastring, "</tr><tr>";
    foreach $y(@y){
       $temp=sprintf "%5.2f",(($sum_sizes{$y}/$sum)*100);
#      $datastring=join '',$datastring, "<td>". int(($sum_sizes{$y}/$sum)*100)."</td>"; # $sum has bee built before
      $datastring=join '',$datastring, "<td>$temp</td>"; # $sum has bee built before
    }
    $datastring=join '',$datastring, "</tr></table><br>";
    $datastring=join '',$datastring, "Overall there are ".int($sum/1000000)." mb of files.<br><br>";
    ###############



    $datastring=join '',$datastring, "<br>Generated by f2html.pl on ".&Date.", (c)2000-2001 Nikolei Steinhage </center></body></html>"
;


    print {STATOUT} "$datastring";

    
    &Closefile(STATOUT);

}


#********************************************************************************
# todo:
# use per2exe to make exe for win --> no
# if last char of defined dir = \ or /, then delete it
# introduce variable for first characters to put in misc category: to define it in a flexible way
# write html-header for firstcharacters that exist!!!
# [.html  is not put into the misc category. "..html" and "x" are hidden links!!!!
# soundex, term::cap, id3fix, renaming of files, seperate tmls for directories
# sql, own sort table, groupings
# rename file2html into f2html
# kmp3 wird als mp3 file erkannt -> falsch
# option fuer direct write and buffered write!!!! sonst evtl speicherprobeme !!!!!!!!!!!!

